/*
 * Copyright (c) 2003 Greg Taleck <taleck@oz.net>
 * 
 * $Id: cksum.c,v 1.2 2003/10/01 18:58:15 taleck Exp $
 *
 */
#include <sys/types.h>
#include <sys/param.h>
#include <sys/time.h>
#include <string.h>

#include "dnet.h"

struct ip6_pseudohdr
{
    struct in6_addr src;
    struct in6_addr dst;
    u_int32_t len;
    u_int32_t proto;
} __attribute__((packed));

struct ip_pseudohdr
{
    struct in_addr src;
    struct in_addr dst;
    u_char  zero;
    u_char proto;
    u_short len;
} __attribute__((packed));

u_int
in_cksum_hdr(const struct ip_hdr *ip)
{
    int i;
    u_int32_t sum = 0;
    u_int16_t *data = (u_int16_t *)ip;

    for (i = 0; i < 10; i++) 
        sum += data[i];
    
    sum = (sum & 0xffff) + (sum >> 16);
    if (sum > 0xffff)
        sum -= 0xffff;

    return ~sum & 0xffff;
}

u_short
in_cksum2(struct ip_hdr *ip, u_int8_t *data)
{
    int i, len;
    u_int32_t sum = 0;
    struct ip_pseudohdr *hdr;
    u_int8_t pkt[2048];

    bzero(pkt, 2048);
    hdr = (struct ip_pseudohdr *)pkt;

    memcpy(&hdr->src, &ip->ip_src, sizeof(struct in_addr));
    memcpy(&hdr->dst, &ip->ip_dst, sizeof(struct in_addr));

    len = ntohs(ip->ip_len) - sizeof(struct ip_hdr);

    if (len < 0)
        return 0;
    
    hdr->len = htons(len);

    hdr->proto = ip->ip_p;
    memcpy(pkt + sizeof(struct ip_pseudohdr), data, len);
    len+= sizeof(struct ip_hdr);
    for (i = 0; i < len; i+=2) {
        sum += *(u_short*)&pkt[i];
    }

    if (sum > 0xffff)
        sum = (sum >> 16) + sum;
    
    sum = ~sum;
    return (u_short)sum;
}



u_short
in6_cksum(struct ip6_hdr *ip6, u_int8_t *data, u_int32_t proto)
{
    u_int sum = 0;
    int i, len;
    struct ip6_pseudohdr *hdr;
    u_char pkt[2048];

    bzero(pkt, 2048);
    hdr = (struct ip6_pseudohdr *)pkt;

    memcpy(&hdr->src, &ip6->ip6_src.data, sizeof(struct in6_addr));
    memcpy(&hdr->dst, &ip6->ip6_dst.data, sizeof(struct in6_addr));
    len = ntohs(ip6->ip6_plen);

    hdr->len = ip6->ip6_plen;
    hdr->proto = htonl(proto);
    memcpy(pkt + sizeof(struct ip6_pseudohdr), data, len);
    len += sizeof(struct ip6_hdr);
    for (i = 0; i < len; i+=2) {
        sum += *(u_short *)&pkt[i];
    }

    if (sum > 0xffff)
        sum = (sum >> 16) + sum;
    
    sum = ~sum;
    return (u_short)sum;
}

