/*
 * Copyright (c) 2003 Greg Taleck <taleck@oz.net>
 * 
 * $Id: misc.c,v 1.2 2003/10/01 18:58:15 taleck Exp $
 */
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "dnet.h"

/*
 * resolve_ip6() returns 0 if successful and
 * addr is initialized; -1 otherwise
 */
int
resolve_ip6(struct ip6_addr *addr, const char *name)
{
    char addrstr[512];
    struct addrinfo *res0 = NULL;
    struct addrinfo *res = NULL;
    struct addrinfo *last_res = NULL;
    struct addrinfo hints;
    /*
     * first try ip6 addresses in ascii fmt
     */
    if (inet_pton(AF_INET6, name, addr) == 1) {
        /* parsed successfully */
        return 0;
    }

    /*
     * failed. now try a lookup
     */
    bzero(&hints, sizeof(hints));
    hints.ai_family = PF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    if (getaddrinfo(name, NULL, &hints, &res0) == 0) {
        /* try to find an AF_INET6 */
        for (res = res0; res; res = res->ai_next) {

            if (res->ai_family == PF_INET6) {
                inet_ntop(res->ai_family,
                          &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr,
                          addrstr, sizeof(addrstr));
                printf("   + found IPv6 address: %s\n", addrstr);
                memcpy(addr,
                       &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr,
                       sizeof(struct in6_addr));
                freeaddrinfo(res0);
                return 0;
            } else if (res->ai_family == PF_INET) {
                inet_ntop(res->ai_family,
                          &((struct sockaddr_in *)res->ai_addr)->sin_addr,
                          addrstr, sizeof(addrstr));
                printf("   + found %s\n", addrstr);
                last_res = res;
            }
        }
    }

    if (last_res) {
        struct sockaddr_in *sa = (struct sockaddr_in *)last_res->ai_addr;
        printf("   + no IPv6 addresses found for %s\n", name);
        printf("   + using %s in IPv4-mapped IPv6 address\n", addrstr);

        ((u_int32_t *)addr->data)[0] = 0;
        ((u_int32_t *)addr->data)[1] = 0;
        ((u_int32_t *)addr->data)[2] = 0xffff0000;
        ((u_int32_t *)addr->data)[3] = sa->sin_addr.s_addr;
        
        freeaddrinfo(res0);
        return 0;
    }

    /* failed too. we're fucked. */
    return -1;
}
